import type { GroupWithTasks, TaskWithSteps, StepWithNotes } from '../types';

interface GraphQLStep {
  id: number;
  name: string;
  instructions: string;
  taskId: number;
  order: number;
  notes: GraphQLNote[];
  print_count: number;
  last_printed_at: string | null;
}

interface GraphQLNote {
  id: number;
  content: string;
  created_at: string;
  updated_at: string;
  user: {
    id: number;
    name: string;
  };
}

interface GraphQLTask {
  id: number;
  name: string;
  groupId: number;
  steps: GraphQLStep[];
  notes: GraphQLNote[];
  print_count: number;
  last_printed_at: string | null;
}

interface GraphQLGroup {
  id: number;
  name: string;
  tasks: GraphQLTask[];
  created_at: string;
  updated_at: string;
}

// Generic type guard for non-null values
export function doesExist<T>(value: T | null | undefined): value is T {
  return value !== null && value !== undefined;
}

// Type guard for arrays
export function isArray<T>(value: unknown): value is T[] {
  return Array.isArray(value);
}

// Type guard for objects with required properties
export function hasProperties<T extends object>(value: unknown, properties: (keyof T)[]): value is T {
  return (
    typeof value === 'object' &&
    value !== null &&
    properties.every(prop => prop in value)
  );
}

// Type guard for GraphQL objects
export function isGraphQLStep(value: unknown): value is GraphQLStep {
  return (
    typeof value === 'object' &&
    value !== null &&
    'id' in value &&
    'name' in value &&
    'instructions' in value &&
    'taskId' in value &&
    'order' in value &&
    'notes' in value &&
    'print_count' in value &&
    'last_printed_at' in value
  );
}

export function isGraphQLTask(value: unknown): value is GraphQLTask {
  return (
    typeof value === 'object' &&
    value !== null &&
    'id' in value &&
    'name' in value &&
    'groupId' in value &&
    'steps' in value &&
    'notes' in value &&
    'print_count' in value &&
    'last_printed_at' in value
  );
}

export function isGraphQLGroup(value: unknown): value is GraphQLGroup {
  return (
    typeof value === 'object' &&
    value !== null &&
    'id' in value &&
    'name' in value &&
    'tasks' in value &&
    'created_at' in value &&
    'updated_at' in value
  );
}

// Transformation functions
export function toStepWithNotes(step: GraphQLStep | null | undefined): StepWithNotes | null {
  if (!doesExist(step)) return null;

  return {
    id: step.id,
    name: step.name,
    instructions: step.instructions,
    task_id: step.taskId,
    order: step.order,
    print_count: step.print_count,
    last_printed_at: step.last_printed_at ? new Date(step.last_printed_at) : undefined,
    notes: isArray(step.notes) ? step.notes.map(note => ({
      id: note.id,
      content: note.content,
      created_at: new Date(note.created_at),
      updated_at: new Date(note.updated_at),
      created_by: note.user.id,
      user: {
        id: note.user.id,
        name: note.user.name,
      },
    })) : [],
  };
}

export function toTaskWithSteps(task: GraphQLTask | null | undefined): TaskWithSteps | null {
  if (!doesExist(task)) return null;

  return {
    id: task.id,
    name: task.name,
    group_id: task.groupId,
    print_count: task.print_count,
    last_printed_at: task.last_printed_at ? new Date(task.last_printed_at) : undefined,
    steps: isArray(task.steps) ? task.steps.map(step => toStepWithNotes(step)).filter(doesExist) : [],
    notes: isArray(task.notes) ? task.notes.map(note => ({
      id: note.id,
      content: note.content,
      created_at: new Date(note.created_at),
      updated_at: new Date(note.updated_at),
      created_by: note.user.id,
      user: {
        id: note.user.id,
        name: note.user.name,
      },
    })) : [],
  };
}

export function toGroupWithTasks(group: GraphQLGroup | null | undefined): GroupWithTasks | null {
  if (!doesExist(group)) return null;

  return {
    id: group.id,
    name: group.name,
    created_at: new Date(group.created_at),
    updated_at: new Date(group.updated_at),
    tasks: isArray(group.tasks) ? group.tasks.map(task => toTaskWithSteps(task)).filter(doesExist) : [],
  };
}